from m5 import *
import os
from shutil import copy, copytree
import glob

# Originally written by James Srinivasan
# Modified by Marius Grannaes, 2006, <grannas FORK idi.ntnu.no>
# Further modified by Magnus Jahre, 2006 <jahre FORK idi.ntnu.no>

# Assumes current working directory is where we ought to run the benchmarks from, copy datasets to etc.

rootdir = os.getenv("DIPPROOT")
if rootdir == None:
  print "Envirionment variable DIPPROOT not set. Quitting..."
  sys.exit(-1)

# Root for lgred
lgred_root = rootdir+'/experiments/benchmarks/spec2000/SPEC_2000_REDUCED'

# Location of SPEC binaries
spec_bin  = rootdir+'/experiments/benchmarks/spec2000'


############################################################

class Gzip(LiveProcess):
  executable = os.path.join(spec_bin, 'gzip00.peak.ev6')

class GzipLgred(Gzip):

  def __init__(self, _value_parent = None, **kwargs):
    Gzip.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '164.gzip/input/lgred.log') , '.')

  cmd = 'gzip00.peak.ev6 lgred.log 1'

############################################################

class Gcc(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
  executable = os.path.join(spec_bin, 'gcc00.peak.ev6')

class GccLgred(Gcc):
  def __init__(self, _value_parent = None, **kwargs):
    Gcc.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '176.gcc/input/lgred.cp-decl.i') , ".")
    
  cmd = 'gcc00.peak.ev6 lgred.cp-decl.i -o lgred.cp-decl.s'


############################################################

class Mcf(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    #copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '181.mcf/data/ref/input/inp.in') , ".")
    
    executable = os.path.join(spec_bin, 'mcf00.peak.ev6')
  cmd = 'mcf00.peak.ev6 inp.in'

class McfLgred(Mcf):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '181.mcf/input/lgred.in') , ".")
    
    executable = os.path.join(spec_bin, 'mcf00.peak.ev6')
  cmd = 'mcf00.peak.ev6 lgred.in'


############################################################

class Crafty(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '186.crafty/data/ref/input/crafty.in') , ".")
    
  executable = os.path.join(spec_bin, 'crafty00.peak.ev6')
  cmd = 'crafty00.peak.ev6 ' 
  input = 'crafty.in'    # source for stderr

class CraftyLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '186.crafty/input/lgred/lgred.in') , ".")
    
  executable = os.path.join(spec_bin, 'crafty00.peak.ev6')
  cmd = 'crafty00.peak.ev6 ' 
  input = 'lgred.in'    # source for stderr

############################################################

class Bzip2(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
  executable = os.path.join(spec_bin, 'bzip200.peak.ev6')

class Bzip2Lgred(Bzip2):
  def __init__(self, _value_parent = None, **kwargs):
    Bzip2.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '256.bzip2/input/lgred.source') , ".")
      
  cmd = 'bzip200.peak.ev6 lgred.source 1'


############################################################

class Wupwise(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '168.wupwise/data/ref/input/wupwise.in') , ".")
        
  executable = os.path.join(spec_bin, 'wupwise00.peak.ev6')
  cmd = 'wupwise00.peak.ev6' 

class WupwiseLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '168.wupwise/input/lgred/wupwise.in') , ".")
        
  executable = os.path.join(spec_bin, 'wupwise00.peak.ev6')
  cmd = 'wupwise00.peak.ev6' 

############################################################

class Swim(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '171.swim/data/ref/input/swim.in') , ".")
        
  executable = os.path.join(spec_bin, 'swim00.peak.ev6')
  cmd = 'swim00.peak.ev6' 
  input = 'swim.in'

class SwimLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '171.swim/input/lgred/swim.in') , ".")
        
  executable = os.path.join(spec_bin, 'swim00.peak.ev6')
  cmd = 'swim00.peak.ev6' 
  input = 'swim.in'

############################################################

class Mgrid(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '172.mgrid/data/ref/input/mgrid.in') , ".")
        
  executable = os.path.join(spec_bin, 'mgrid00.peak.ev6')
  cmd = 'mgrid00.peak.ev6' 
  input = 'mgrid.in'

class MgridLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '172.mgrid/input/lgred/mgrid.in') , ".")
        
  executable = os.path.join(spec_bin, 'mgrid00.peak.ev6')
  cmd = 'mgrid00.peak.ev6' 
  input = 'mgrid.in'

############################################################

class Applu(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '173.applu/data/ref/input/applu.in') , ".")
        
  executable = os.path.join(spec_bin, 'applu00.peak.ev6')
  cmd = 'applu00.peak.ev6' 
  input = 'applu.in'

class AppluLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '173.applu/input/lgred/applu.in') , ".")
        
  executable = os.path.join(spec_bin, 'applu00.peak.ev6')
  cmd = 'applu00.peak.ev6' 
  input = 'applu.in'

############################################################

class Mesa(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
       
    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '177.mesa/data/ref/input/mesa.in') , ".")
    #copy(os.path.join(spec_root, '177.mesa/data/ref/input/numbers') , ".")
        
  executable = os.path.join(spec_bin, 'mesa00.peak.ev6')
  cmd = 'mesa00.peak.ev6 -frames 1000 -meshfile mesa.in -ppmfile mesa.ppm' 

class MesaLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
       
    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '177.mesa/input/lgred.in') , ".")
        
  executable = os.path.join(spec_bin, 'mesa00.peak.ev6')
  cmd = 'mesa00.peak.ev6 -frames 1000 -meshfile lgred.in -ppmfile mesa.ppm' 

############################################################

class Galgel(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '178.galgel/data/ref/input/galgel.in') , ".")
          
  executable = os.path.join(spec_bin, 'galgel00.peak.ev6')
  cmd = 'galgel00.peak.ev6'
  input = 'galgel.in'

class GalgelLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '178.galgel/input/lgred/lgred.in') , ".")
          
  executable = os.path.join(spec_bin, 'galgel00.peak.ev6')
  cmd = 'galgel00.peak.ev6'
  input = 'lgred.in'

############################################################

class Equake(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '183.equake/data/ref/input/inp.in') , ".")
          
  executable = os.path.join(spec_bin, 'equake00.peak.ev6')
  cmd = 'equake00.peak.ev6'
  input = 'inp.in'

class EquakeLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '183.equake/input/lgred/lgred.in') , ".")
          
  executable = os.path.join(spec_bin, 'equake00.peak.ev6')
  cmd = 'equake00.peak.ev6'
  input = 'lgred.in'



############################################################

class Lucas(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '189.lucas/data/ref/input/lucas2.in') , ".")
          
  executable = os.path.join(spec_bin, 'lucas00.peak.ev6')
  cmd = 'lucas00.peak.ev6'
  input = 'lucas2.in'

class LucasLgred(LiveProcess):
  def __init__(self, _value_parent = None, **kwargs):
    LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    # copy input file(s) to run directory  
    copy(os.path.join(lgred_root, '189.lucas/input/lgred/lgred.in') , ".")
          
  executable = os.path.join(spec_bin, 'lucas00.peak.ev6')
  cmd = 'lucas00.peak.ev6'
  input = 'lgred.in'

###########################################################

#TODO: add lgred support

#class Fma3d(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '191.fma3d/data/ref/input/fma3d.in') , ".")
          
  #executable = os.path.join(spec_bin, 'fma3d00.peak.ev6')
  #cmd = 'fma3d00.peak.ev6'

############################################################

#TODO: add lgred support

#class Sixtrack(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file by file
    #for file in glob.glob(os.path.join(spec_root, '200.sixtrack/data/all/input/*')):
        #copy(file, ".")
    #for file in glob.glob(os.path.join(spec_root, '200.sixtrack/data/ref/input/*')):
        #copy(file, ".")     
    
  #executable = os.path.join(spec_bin, 'sixtrack00.peak.ev6')
  #cmd = 'sixtrack00.peak.ev6'    
  #input = 'inp.in'
    
############################################################

#TODO: add lgred support

#class Apsi(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '301.apsi/data/ref/input/apsi.in') , ".")
          
  #executable = os.path.join(spec_bin, 'apsi00.peak.ev6')
  #cmd = 'apsi00.peak.ev6'    
  #input = 'apsi.in'
  
#TODO: add lgred input for parser

#class Parser(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '197.parser/data/ref/input/ref.in') ,   ".")
    #copy(os.path.join(spec_root, '197.parser/data/all/input/2.1.dict') , ".")
    
    ## for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
    #if not os.path.exists("words"):
        #copytree(os.path.join(spec_root, '197.parser/data/all/input/words') , "words")
    
  #executable = os.path.join(spec_bin, 'parser00.peak.ev6')
  #cmd = 'parser00.peak.ev6 2.1.dict -batch' 
  #input = 'ref.in'     # source for stderr

############################################################

#TODO : add lgred support for Eon

#class Eon(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/eon.dat'), ".")
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/materials'), ".")
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/spectra.dat'), ".")
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/chair.camera'), ".")
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/chair.surfaces'), ".")

  #executable = os.path.join(spec_bin, 'eon00.peak.ev6')

#class Eon1(Eon):
  #def __init__(self, _value_parent = None, **kwargs):
    #Eon.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/chair.control.cook'),".")
    
  #cmd = 'eon00.peak.ev6 chair.control.cook chair.camera chair.surfaces chair.cook.ppm ppm pixels_out.cook' 

#class Eon2(Eon):
  #def __init__(self, _value_parent = None, **kwargs):
    #Eon.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/chair.control.rushmeier') ,  ".")
    
  #cmd = 'eon00.peak.ev6 chair.control.rushmeier chair.camera chair.surfaces chair.rushmeier.ppm ppm pixels_out.rushmeier' 

#class Eon3(Eon):
  #def __init__(self, _value_parent = None, **kwargs):
    #Eon.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '252.eon/data/ref/input/chair.control.kajiya') ,  ".")
  #cmd = 'eon00.peak.ev6 chair.control.kajiya chair.camera chair.surfaces chair.kajiya.ppm ppm pixels_out.kajiya' 
    

############################################################

#TODO: use lgred input files for Perlbmk

#class Perlbmk(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/lenums') , ".")

    ## for some reason this constructor gets called twice but 
    ##if the target already exists copytree will fail so check first
    #if not os.path.exists("lib"):
      #copytree(os.path.join(spec_root, '253.perlbmk/data/all/input/lib') , "lib")

  #executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')

#class Perlbmk1(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/diffmail.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib diffmail.pl 2 550 15 24 23 100' 

#class Perlbmk2(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/ref/input/makerand.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib makerand.pl' 

#class Perlbmk3(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/perfect.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib perfect.pl b 3 m 4' 
    
#class Perlbmk4(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/ref/input/splitmail.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 850 5 19 18 1500'     
    
#class Perlbmk5(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/ref/input/splitmail.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 704 12 26 16 836'

#class Perlbmk6(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/ref/input/splitmail.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 535 13 25 24 1091'
    
#class Perlbmk7(Perlbmk):
  #def __init__(self, _value_parent = None, **kwargs):
    #Perlbmk.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
    #copy(os.path.join(spec_root, '253.perlbmk/data/ref/input/splitmail.pl') , ".")

  #cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 957 12 23 26 1014'

############################################################

#TODO: add lgred support

#class Twolf(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file by file
    #for file in glob.glob(os.path.join(spec_root, '300.twolf/data/ref/input/*')):
       #copy(file, ".")
      
  #executable = os.path.join(spec_bin, 'twolf00.peak.ev6')
  #cmd = 'twolf00.peak.ev6 ref' 
  
############################################################

# TODO: add lgred support

#class Gap(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '254.gap/data/ref/input/ref.in') , ".")
    
    ## copy input file by file
    #for file in glob.glob(os.path.join(spec_root, '254.gap/data/all/input/*')):
       #copy(file, ".")
    
  #executable = os.path.join(spec_bin, 'gap00.peak.ev6')
  #cmd = 'gap00.peak.ev6 -l ./ -q -m 192M'
  ##input = 'ref.in'
  
############################################################

# add lgred support

#class Vortex(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/persons.1k') , ".")
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/lendian.rnv') , ".")
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/lendian.wnv') , ".")

  #executable = os.path.join(spec_bin, 'vortex00.peak.ev6')

#class Vortex1(Vortex):
  #def __init__(self, _value_parent = None, **kwargs):
    #Vortex.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/lendian1.raw') , ".")
      
  #cmd = 'vortex00.peak.ev6 lendian1.raw'

#class Vortex2(Vortex):
  #def __init__(self, _value_parent = None, **kwargs):
    #Vortex.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/lendian2.raw') , ".")
      
  #cmd = 'vortex00.peak.ev6 lendian2.raw'

#class Vortex3(Vortex):
  #def __init__(self, _value_parent = None, **kwargs):
    #Vortex.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '255.vortex/data/ref/input/lendian3.raw') , ".")
      
  #cmd = 'vortex00.peak.ev6 lendian3.raw'
  
  ############################################################

# TODO: add lgred support

#class Art(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file(s) to run directory  
    #copy(os.path.join(spec_root, '179.art/data/ref/input/a10.img')    , ".")
    #copy(os.path.join(spec_root, '179.art/data/ref/input/c756hel.in') , ".")
    #copy(os.path.join(spec_root, '179.art/data/ref/input/hc.img')     , ".")
          
  #executable = os.path.join(spec_bin, 'art00.peak.ev6')

#class Art1(Art):
  #def __init__(self, _value_parent = None, **kwargs):
    #Art.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

  #cmd = 'art00.peak.ev6 -scanfile c756hel.in -trainfile1 a10.img -trainfile2 hc.img -stride 2 -startx 110 -starty 200 -endx 160 -endy 240 -objects 10'

#class Art2(Art):
  #def __init__(self, _value_parent = None, **kwargs):
    #Art.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

  #cmd = 'art00.peak.ev6 -scanfile c756hel.in -trainfile1 a10.img -trainfile2 hc.img -stride 2 -startx 470 -starty 140 -endx 520 -endy 180 -objects 10'
  
  
############################################################

#TODO: add lgred support

#class Facerec(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file by file
    #for file in glob.glob(os.path.join(spec_root, '187.facerec/data/ref/input/*')):
       #copy(file, ".")
    #for file in glob.glob(os.path.join(spec_root, '187.facerec/data/all/input/*')):
       #copy(file, ".")
            
  #executable = os.path.join(spec_bin, 'facerec00.peak.ev6')
  #cmd = 'facerec00.peak.ev6'
  #input = 'ref.in'    
  
  
  
############################################################

# TODO:fix lgred support

#class Ammp(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ## copy input file by file
    #for file in glob.glob(os.path.join(spec_root, '188.ammp/data/ref/input/*')):
       #copy(file, ".")
             
  #executable = os.path.join(spec_bin, 'ammp00.peak.ev6')
  #cmd = 'ammp00.peak.ev6'
  #input = 'ammp.in'
  
############################################################

#TODO: implement lgred support for vpr

#class Vpr(LiveProcess):
  #def __init__(self, _value_parent = None, **kwargs):
    #LiveProcess.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    #executable = os.path.join(spec_bin, 'vpr00.peak.ev6')

#class VprPlace(Vpr):
  #def __init__(self, _value_parent = None, **kwargs):
    #Vpr.__init__(self, _value_parent = None, **kwargs)  # call parent constructor
    ## copy input file(s) to run directory  
    #copy(os.path.join(lgred_root, '175.vpr/data/ref/input/net.in') ,  '.')
    #copy(os.path.join(lgred_root, '175.vpr/data/ref/input/arch.in') , '.')

  #cmd = 'vpr00.peak.ev6 ' +                     \
    #'net.in arch.in place.out dum.out ' +            \
    #'-nodisp -place_only -init_t 5 -exit_t 0.005 -alpha_t 0.9412 -inner_num 2'

##class VprRoute(Vpr):
  ##def __init__(self, _value_parent = None, **kwargs):
    ##Vpr.__init__(self, _value_parent = None, **kwargs)  # call parent constructor

    ### copy input file(s) to run directory  
    ##copy(os.path.join(spec_root, '175.vpr/data/ref/input/place.in') , '.')
    ##copy(os.path.join(spec_root, '175.vpr/data/ref/input/net.in') ,  '.')
    ##copy(os.path.join(spec_root, '175.vpr/data/ref/input/arch.in') , '.')

  ##cmd = 'vpr00.peak.ev6 ' +                              \
      ##'net.in arch.in place.in route.out ' +                    \
    ##'-nodisp -route_only -route_chan_width 15 -pres_fac_mult 2 -acc_fac 1 ' +    \
        ##'-first_iter_pres_fac 4 -initial_pres_fac 8'
