from m5 import *
import os
from shutil import copy, copytree
import glob

# Assumes current working directory is where we ought to run the benchmarks from, copy datasets to etc.

# Root of where SPEC2000 install lives
spec_root = '/anfs/bigdisc/jrs53/spec2000'

# Location of SPEC binaries
spec_bin  = '/anfs/bigdisc/jrs53/m5/alpha/bin'


########################################################################################################################

class GzipSource(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/164.gzip/data/ref/input/input.source') , '.')
	
    executable = os.path.join(spec_bin, 'gzip00.peak.ev6')
    cmd = 'gzip00.peak.ev6 input.source 60'
    
class GzipLog(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/164.gzip/data/ref/input/input.log') , '.')
	
    executable = os.path.join(spec_bin, 'gzip00.peak.ev6')
    cmd = 'gzip00.peak.ev6 input.log 60'

class GzipGraphic(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/164.gzip/data/ref/input/input.graphic') , '.')
	
    executable = os.path.join(spec_bin, 'gzip00.peak.ev6')
    cmd = 'gzip00.peak.ev6 input.graphic 60'

class GzipRandom(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/164.gzip/data/ref/input/input.random') , '.')
	
    executable = os.path.join(spec_bin, 'gzip00.peak.ev6')
    cmd = 'gzip00.peak.ev6 input.random 60'

class GzipProgram(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/164.gzip/data/ref/input/input.program') , '.')
	
    executable = os.path.join(spec_bin, 'gzip00.peak.ev6')
    cmd = 'gzip00.peak.ev6 input.program 60'

########################################################################################################################

class VprPlace(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/175.vpr/data/ref/input/net.in') ,  '.')
	copy(os.path.join(spec_root, 'benchspec/CINT2000/175.vpr/data/ref/input/arch.in') , '.')
	
    executable = os.path.join(spec_bin, 'vpr00.peak.ev6')
    cmd = 'vpr00.peak.ev6 ' + 										\
		  'net.in arch.in place.out dum.out ' +						\
		  '-nodisp -place_only -init_t 5 -exit_t 0.005 -alpha_t 0.9412 -inner_num 2'

class VprRoute(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
        copy(os.path.join(spec_root, 'benchspec/CINT2000/175.vpr/data/ref/input/net.in') ,   '.')
        copy(os.path.join(spec_root, 'benchspec/CINT2000/175.vpr/data/ref/input/arch.in') ,  '.')
        copy(os.path.join(spec_root, 'benchspec/CINT2000/175.vpr/data/ref/input/place.in') , '.')
	
    executable = os.path.join(spec_bin, 'vpr00.peak.ev6')
    cmd = 'vpr00.peak.ev6 ' +															\
    	  'net.in arch.in place.in route.out ' +										\
		  '-nodisp -route_only -route_chan_width 15 -pres_fac_mult 2 -acc_fac 1 ' +		\
          '-first_iter_pres_fac 4 -initial_pres_fac 8'

########################################################################################################################

class Gcc166(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/176.gcc/data/ref/input/166.i') , ".")
	
    executable = os.path.join(spec_bin, 'cc100.peak.ev6')
    cmd = 'cc100.peak.ev6 166.i -o 166.s'

class Gcc200(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/176.gcc/data/ref/input/200.i') , ".")
	
    #executable = os.path.join(spec_bin, 'cc100.peak.ev6')
    #cmd = 'cc100.peak.ev6 200.i -o 200.s'
    executable = os.path.join(spec_bin, 'cc1_peak.alpha-linux')
    cmd = 'cc1_peak.alpha-linux 200.i -o 200.s'
    

class GccExpr(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/176.gcc/data/ref/input/expr.i') , ".")
	
    executable = os.path.join(spec_bin, 'cc100.peak.ev6')
    cmd = 'cc100.peak.ev6 expr.i -o expr.s'

class GccIntegrate(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/176.gcc/data/ref/input/integrate.i') , ".")
	
    executable = os.path.join(spec_bin, 'cc100.peak.ev6')
    cmd = 'cc100.peak.ev6 integrate.i -o integrate.s'

class GccScilab(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/176.gcc/data/ref/input/scilab.i') , ".")
	
    executable = os.path.join(spec_bin, 'cc100.peak.ev6')
    cmd = 'cc100.peak.ev6 scilab.i -o scilab.s'

########################################################################################################################

class Mcf(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/181.mcf/data/ref/input/inp.in') , ".")
	
    executable = os.path.join(spec_bin, 'mcf00.peak.ev6')
    cmd = 'mcf00.peak.ev6 inp.in'

########################################################################################################################

class Crafty(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/186.crafty/data/ref/input/crafty.in') , ".")
	
    executable = os.path.join(spec_bin, 'crafty00.peak.ev6')
    cmd = 'crafty00.peak.ev6 ' 
    input = 'crafty.in'		# source for stderr

########################################################################################################################

class Parser(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/197.parser/data/ref/input/ref.in') ,   ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/197.parser/data/all/input/2.1.dict') , ".")
	
	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("words"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/197.parser/data/all/input/words') , "words")
	
    executable = os.path.join(spec_bin, 'parser00.peak.ev6')
    cmd = 'parser00.peak.ev6 2.1.dict -batch' 
    input = 'ref.in' 		# source for stderr

########################################################################################################################

class Eon1(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/eon.dat') ,             ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/materials') ,           ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/spectra.dat') ,         ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.control.cook') ,  ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.camera') ,        ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.surfaces') ,       ".")
	
    executable = os.path.join(spec_bin, 'eon00.peak.ev6')
    cmd = 'eon00.peak.ev6 chair.control.cook chair.camera chair.surfaces chair.cook.ppm ppm pixels_out.cook' 

class Eon2(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/eon.dat') ,             ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/materials') ,           ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/spectra.dat') ,         ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.control.rushmeier') ,  ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.camera') ,        ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.surfaces') ,       ".")
	
    executable = os.path.join(spec_bin, 'eon00.peak.ev6')
    cmd = 'eon00.peak.ev6 chair.control.rushmeier chair.camera chair.surfaces chair.rushmeier.ppm ppm pixels_out.rushmeier' 

class Eon3(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/eon.dat') ,             ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/materials') ,           ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/spectra.dat') ,         ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.control.kajiya') ,  ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.camera') ,        ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/252.eon/data/ref/input/chair.surfaces') ,       ".")
	
    executable = os.path.join(spec_bin, 'eon00.peak.ev6')
    cmd = 'eon00.peak.ev6 chair.control.kajiya chair.camera chair.surfaces chair.kajiya.ppm ppm pixels_out.kajiya' 
    

########################################################################################################################


class Perlbmk1(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/diffmail.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib diffmail.pl 2 550 15 24 23 100' 

class Perlbmk2(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/ref/input/makerand.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib makerand.pl' 

class Perlbmk3(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/perfect.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib perfect.pl b 3 m 4' 
    
class Perlbmk4(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/ref/input/splitmail.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 850 5 19 18 1500'     
    
class Perlbmk5(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/ref/input/splitmail.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 704 12 26 16 836'

class Perlbmk6(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/ref/input/splitmail.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 535 13 25 24 1091'
    
class Perlbmk7(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lenums') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/cpu2000_mhonarc.rc') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/ref/input/splitmail.pl') , ".")

	# for some reason this constructor gets called twice but if the target already exists copytree will fail so check first
	if not os.path.exists("lib"):
	    copytree(os.path.join(spec_root, 'benchspec/CINT2000/253.perlbmk/data/all/input/lib') , "lib")

	
    executable = os.path.join(spec_bin, 'perlbmk00.peak.ev6')
    cmd = 'perlbmk00.peak.ev6 -I./lib splitmail.pl 957 12 23 26 1014'
    
########################################################################################################################

class Gap(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/254.gap/data/ref/input/ref.in') , ".")
	
	# copy input file by file
	for file in glob.glob(os.path.join(spec_root, 'benchspec/CINT2000/254.gap/data/all/input/*')):
	 	copy(file, ".")
	
    executable = os.path.join(spec_bin, 'gap00.peak.ev6')
    cmd = 'gap00.peak.ev6 -l ./ -q -m 192M'
    input = 'ref.in'
    
########################################################################################################################

class Vortex1(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/persons.1k') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.rnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.wnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian1.raw') , ".")
		
    executable = os.path.join(spec_bin, 'vortex00.peak.ev6')
    cmd = 'vortex00.peak.ev6 lendian1.raw'

class Vortex2(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/persons.1k') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.rnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.wnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian2.raw') , ".")
		
    executable = os.path.join(spec_bin, 'vortex00.peak.ev6')
    cmd = 'vortex00.peak.ev6 lendian2.raw'

class Vortex3(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/persons.1k') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.rnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian.wnv') , ".")
	copy(os.path.join(spec_root, 'benchspec/CINT2000/255.vortex/data/ref/input/lendian3.raw') , ".")
		
    executable = os.path.join(spec_bin, 'vortex00.peak.ev6')
    cmd = 'vortex00.peak.ev6 lendian3.raw'

########################################################################################################################

class Bzip2Source(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/256.bzip2/data/ref/input/input.source') , ".")
		
    executable = os.path.join(spec_bin, 'bzip200.peak.ev6')
    cmd = 'bzip200.peak.ev6 input.source 58'

class Bzip2Graphic(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/256.bzip2/data/ref/input/input.graphic') , ".")
		
    executable = os.path.join(spec_bin, 'bzip200.peak.ev6')
    cmd = 'bzip200.peak.ev6 input.graphic 58'
    
class Bzip2Program(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CINT2000/256.bzip2/data/ref/input/input.program') , ".")
		
    executable = os.path.join(spec_bin, 'bzip200.peak.ev6')
    cmd = 'bzip200.peak.ev6 input.program 58' 

########################################################################################################################

class Twolf(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):

    	LiveProcess.__init__(self)	# call parent constructor
	
	# copy input file by file
	for file in glob.glob(os.path.join(spec_root, 'benchspec/CINT2000/300.twolf/data/ref/input/*')):
	 	copy(file, ".")
		
    executable = os.path.join(spec_bin, 'twolf00.peak.ev6')
    cmd = 'twolf00.peak.ev6 ref' 


########################################################################################################################
########################################################################################################################

class Wupwise(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/168.wupwise/data/ref/input/wupwise.in') , ".")
			
    executable = os.path.join(spec_bin, 'wupwise00.peak.ev6')
    cmd = 'wupwise00.peak.ev6' 

########################################################################################################################

class Swim(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/171.swim/data/ref/input/swim.in') , ".")
			
    executable = os.path.join(spec_bin, 'swim00.peak.ev6')
    cmd = 'swim00.peak.ev6' 
    input = 'swim.in'

########################################################################################################################

class Mgrid(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/172.mgrid/data/ref/input/mgrid.in') , ".")
			
    executable = os.path.join(spec_bin, 'mgrid00.peak.ev6')
    cmd = 'mgrid00.peak.ev6' 
    input = 'mgrid.in'

########################################################################################################################

class Applu(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/173.applu/data/ref/input/applu.in') , ".")
			
    executable = os.path.join(spec_bin, 'applu00.peak.ev6')
    cmd = 'applu00.peak.ev6' 
    input = 'applu.in'

########################################################################################################################

class Mesa(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/177.mesa/data/ref/input/mesa.in') , ".")
	copy(os.path.join(spec_root, 'benchspec/CFP2000/177.mesa/data/ref/input/numbers') , ".")
			
    executable = os.path.join(spec_bin, 'mesa00.peak.ev6')
    cmd = 'mesa00.peak.ev6 -frames 1000 -meshfile mesa.in -ppmfile mesa.ppm' 

########################################################################################################################

class Galgel(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/178.galgel/data/ref/input/galgel.in') , ".")
				
    executable = os.path.join(spec_bin, 'galgel00.peak.ev6')
    cmd = 'galgel00.peak.ev6'
    input = 'galgel.in'

########################################################################################################################

class Art1(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/a10.img')    , ".")
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/c756hel.in') , ".")
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/hc.img')     , ".")
				
    executable = os.path.join(spec_bin, 'art00.peak.ev6')
    cmd = 'art00.peak.ev6 -scanfile c756hel.in -trainfile1 a10.img -trainfile2 hc.img -stride 2 -startx 110 -starty 200 -endx 160 -endy 240 -objects 10'

########################################################################################################################

class Art2(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/a10.img')    , ".")
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/c756hel.in') , ".")
	copy(os.path.join(spec_root, 'benchspec/CFP2000/179.art/data/ref/input/hc.img')     , ".")
				
    executable = os.path.join(spec_bin, 'art00.peak.ev6')
    cmd = 'art00.peak.ev6 -scanfile c756hel.in -trainfile1 a10.img -trainfile2 hc.img -stride 2 -startx 470 -starty 140 -endx 520 -endy 180 -objects 10'

########################################################################################################################

class Equake(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/183.equake/data/ref/input/inp.in') , ".")
				
    executable = os.path.join(spec_bin, 'equake00.peak.ev6')
    cmd = 'equake00.peak.ev6'
    input = 'inp.in'

########################################################################################################################

class Facerec(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file by file
	for file in glob.glob(os.path.join(spec_root, 'benchspec/CFP2000/187.facerec/data/ref/input/*')):
	 	copy(file, ".")
	for file in glob.glob(os.path.join(spec_root, 'benchspec/CFP2000/187.facerec/data/all/input/*')):
	 	copy(file, ".")
					
    executable = os.path.join(spec_bin, 'facerec00.peak.ev6')
    cmd = 'facerec00.peak.ev6'
    input = 'ref.in'    

########################################################################################################################

class Ammp(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file by file
	for file in glob.glob(os.path.join(spec_root, 'benchspec/CFP2000/188.ammp/data/ref/input/*')):
	 	copy(file, ".")
	 				
    executable = os.path.join(spec_bin, 'ammp00.peak.ev6')
    cmd = 'ammp00.peak.ev6'
    input = 'ammp.in'

########################################################################################################################

class Lucas(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/189.lucas/data/ref/input/lucas2.in') , ".")
				
    executable = os.path.join(spec_bin, 'lucas00.peak.ev6')
    cmd = 'lucas00.peak.ev6'
    input = 'lucas2.in'

########################################################################################################################

class Fma3d(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/191.fma3d/data/ref/input/fma3d.in') , ".")
				
    executable = os.path.join(spec_bin, 'fma3d00.peak.ev6')
    cmd = 'fma3d00.peak.ev6'

########################################################################################################################

class Sixtrack(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
        # copy input file by file
        for file in glob.glob(os.path.join(spec_root, 'benchspec/CFP2000/200.sixtrack/data/all/input/*')):
            copy(file, ".")
        for file in glob.glob(os.path.join(spec_root, 'benchspec/CFP2000/200.sixtrack/data/ref/input/*')):
            copy(file, ".")	 	
				
    executable = os.path.join(spec_bin, 'sixtrack00.peak.ev6')
    cmd = 'sixtrack00.peak.ev6'    
    input = 'inp.in'
    
########################################################################################################################

class Apsi(LiveProcess):

    def __init__(self, _value_parent = None, **kwargs):
        
        LiveProcess.__init__(self)	# call parent constructor  
        
    # copy input file(s) to run directory	
	copy(os.path.join(spec_root, 'benchspec/CFP2000/301.apsi/data/ref/input/apsi.in') , ".")
				
    executable = os.path.join(spec_bin, 'apsi00.peak.ev6')
    cmd = 'apsi00.peak.ev6'    
    input = 'apsi.in'
        
########################################################################################################################
########################################################################################################################

class Helloworld(LiveProcess):

	executable = os.path.join(spec_bin, 'helloworld')
	cmd = 'helloworld'

    
    
        
