
import popen2
import pbsconfig

R_QUIET = True

class Bar_Plotter:
    
    # TODO: Extend barplotter color list
    availColors = ["#FF3300", "#3333FF", "#99FF66", "gray", "#FFFF66"]
    colors = []
    for i in range(len(pbsconfig.protocols) * len(pbsconfig.interconnects)):
        colors.append(availColors[i])
    
    max_y = "*"
    min_y = "*"
    
    if R_QUIET:
        quiet_option = "--quiet"
    else:
        quiet_option = ""
    
    def __init__(self, title, xlabel, ylabel, benchmarks, configs, filepath):
        self.title = title
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.benchmarks = benchmarks
        self.configs = configs
        self.filepath = filepath
        
    def write_data_file(self, data, filename):
        output = open(self.filepath+"/"+filename, "w")
        
        output.write("Benchmark")
        #FIXME: should handle both protocols and interconnects
        #for config in pbsconfig.protocols:
        for config in pbsconfig.interconnects:
            output.write(";"+config);
        output.write("\n")
        
        for line in data:
            name, vals = line
            output.write("\""+str(name)+"\"")
            for val in vals:
                output.write(";"+str(val))
            output.write("\n")
        
        output.close()
        
    def write_script(self, data_file, r_file_name, eps_name, cpu_count):
        output = open(self.filepath+"/"+r_file_name, "w")
        
        output.write("postscript(file=\""+self.filepath+"/"+eps_name+"\", print.it=FALSE, pointsize=12, family=\"AvantGarde\")\n");
        
        output.write("par(mar = c(9, 4, 4, 2))\n")
        
        output.write("data <- read.csv(\""+self.filepath+"/"+data_file+"\", header=TRUE, sep=\";\", dec=\".\", row.names=1)\n")
        output.write("mat <- as.matrix(data)\n")
        output.write("tmat <- t(mat)\n")
        
        first = 1
        output.write("colors <- c(")
        for color in self.colors:
            if first == 1:
                output.write("\""+color+"\"")
                first = 0
            else:
                output.write(","+"\""+color+"\"")
        output.write(")\n")
        
        command = "pts <- barplot(tmat,beside=TRUE,xaxt=\"n\",xlab=\"\", ylab=\""+self.ylabel+"\", main=\""+self.title+" with "+str(cpu_count)+" CPUs\", legend=TRUE, col=colors"
        if self.max_y == "*":
            output.write(command)
        else:
            output.write(command+", ylim=c("+self.min_y+","+self.max_y+")")
        output.write(")\n")
        
        output.write("temp <- array(0, dim=c(0,12))\n")
        output.write("for(i in 1:ncol(pts)) temp[i] <- mean(pts[,i])\n")
    
        output.write("labels <- c(");
        first = 1
        for benchmark in pbsconfig.benchmarks:
            if first == 1:
                first = 0
            else:
                output.write(",")
            output.write("\""+str(benchmark)+"\"")
        output.write(")\n")
        
        # output.write("axis(side=1,at=temp, labels=F, tick=T)\n")
        if self.max_y == "*":
            output.write("text(temp, -0.015*max(data), srt=90, adj=1,labels=labels, xpd=TRUE)\n")
        else:
            output.write("text(temp, -0.015*"+self.max_y+", srt=90, adj=1,labels=labels, xpd=TRUE)\n")
        output.write("mtext(1, text = \""+self.xlabel+"\", line = 8)\n")
        
        output.close()
        
    def plot_bars(self, r_script, eps_name):
        results = popen2.popen3("R --vanilla "+self.quiet_option+" < "+ self.filepath+"/"+r_script)
        #print "R output:"
        #print results[0].read()
        tmp = results[0].read()
        
        results = popen2.popen3("epstopdf "+ self.filepath+"/"+eps_name)
        print results[0].read()
        print results[2].read()
        
