
import popen2

class Box_Plotter:
    # set default values
    min_x = "*"
    max_x = "*"
    min_y = "*"
    max_y = "*"
    
    textwidth = 20
    datawidth = 20
     
    def __init__(self, title, xlabel, ylabel, benchmarks, configs, filepath):
        self.title = title
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.benchmarks = benchmarks
        self.configs = configs
        self.filepath = filepath
        
    
    def write_data_file(self, datafile, data):
        output = open(self.filepath+"/"+datafile, "w")
        
        output.write("#Name".ljust(self.textwidth)+"xtic".ljust(self.datawidth))
        for config in self.configs:
          output.write(config.ljust(self.datawidth))
        output.write("\n")
        
        xtic = 1
        for line in data:
          name, vals = line
          output.write(name.ljust(self.datawidth))
          output.write(str(xtic).ljust(self.datawidth))
        
          for element in vals:
            output.write(str(element).ljust(self.datawidth))
        
          xtic = xtic + 1
          output.write("\n")
        
        output.close()

    def write_benchmark_script(self, datafile, gp_script, epsfile, cpu_count):
        output = open(self.filepath+"/"+gp_script,'w')
        
        output.write("set title \""+self.title+" with "+str(cpu_count)+" CPUs\"\n");
        output.write("set xlabel \""+self.xlabel+"\"\n");
        output.write("set ylabel \""+self.ylabel+"\"\n");
          
        output.write("set terminal postscript eps color enhanced\n");
        output.write("set output \""+self.filepath+"/"+epsfile+"\"\n");
        
        output.write("set xtics rotate (")
        i = 1
        for benchmark in self.benchmarks:
          output.write("\""+benchmark+"\" "+str(i))
          if i == (len(self.benchmarks)):
            output.write(")\n\n")
          else:
            output.write(", ")
          i = i + 1
        
        boxwidth = 0.8 / len(self.configs)
        startpoint = 0 - ((len(self.configs)/2) * boxwidth)
        
        output.write("plot ")
        output.write("["+self.min_x+":"+self.max_x+"] ")
        output.write("["+self.min_y+":"+self.max_y+"] ")
        
        for i in range(0, len(self.configs)):
          config = self.configs[i]
            
          thisStart = startpoint + (i*boxwidth)
          if thisStart < 0:
            thisStartStr = str(thisStart)
          else:
            thisStartStr = "+"+str(thisStart)
          
          output.write("\""+self.filepath+"/"+datafile+"\" ")
          output.write("using ($2"+thisStartStr+"):"+str(i+3)+":("+str(boxwidth)+") ")
          output.write("title \""+config+"\" ")
          output.write("with boxes fs fill pattern "+str(i+1))
          
          if i == (len(self.configs)-1):
            output.write("\n")
          else:
            output.write(", \\\n")
            
        output.close()
    
    def plot_boxes(self, gpscript, pdf_filename):
        
        results = popen2.popen3("gnuplot "+ self.filepath+"/"+gpscript)
        
        print "Gnuplot output: "+results[0].read()
        
        results = popen2.popen3("epstopdf "+ self.filepath+"/"+pdf_filename)
        print "epstopdf output: "+results[0].read()
        
