
from pysqlite2 import dbapi2 as sqlite
import sys
import create_table_queries
import database_constants

# ------ METHODS --------------------------------------------------------------

class Database:
  database_file_name = database_constants.DATABASE_FILE
  current_config_id = 0
  current_benchmark_id = 0
  current_result_id = 0
  current_L2_shared_id = 0
  current_L1_i_id = 0
  current_L1_d_id = 0
  current_shared_bus_id = 0
  current_interconnect_id = 0
  
  def __init__(self):
    self.connection = sqlite.connect(database_constants.DATABASE_FILE)

  def create_database(self):
    cur = self.connection.cursor()
    for tablequery in create_table_queries.create_table_queries:
      try:
        cur.execute(tablequery)
      except sqlite.OperationalError:
        self.panic("The database "+database_constants.DATABASE_FILE+" already exists. "+\
                   "Delete or move it and try again.")
    
    self.connection.commit()

  def get_benchmark_data_length(self):
    return 1

  def add_benchmark(self, data):
    if len(data) != self.get_benchmark_data_length():
      self.panic("The length of the benchmark data array must be "+str(self.get_benchmark_data_length())+". Found length "+len(data))
    cur = self.connection.cursor()
    self.current_benchmark_id = self.current_benchmark_id + 1
    cur.execute('INSERT INTO '+database_constants.BENCHMARK_TABLE_NAME+' VALUES(?,?)',
                 (self.current_benchmark_id,
                  data[0]
                 )
               )

    self.connection.commit()
    return self.current_benchmark_id

  def get_configuration_data_length(self):
    return 6
  
  def add_configuration(self, data):
    
    if len(data) != self.get_configuration_data_length():
      self.panic("The length of the configuration data array must be "+str(self.get_configuration_data_length())+". Found length "+str(len(data)))
    
    cur = self.connection.cursor()
    self.current_config_id = self.current_config_id + 1
    cur.execute('INSERT INTO '+database_constants.CONFIGURATION_TABLE_NAME+' VALUES(?,?,?,?,?,?,?)',
                 (self.current_config_id,
                  data[0],
                  data[1],
                  data[2],
                  data[3],
                  data[4],
                  data[5]
                 )
                )

    self.connection.commit()
    return self.current_config_id
  
  def get_result_data_length(self):
    return 6
  
  def add_result(self, data):
    
    if len(data) != self.get_result_data_length():
      self.panic("The length of the result data array must be "+str(self.get_result_data_length())+". Found length "+str(len(data)))
    
    cur = self.connection.cursor()
    self.current_result_id = self.current_result_id + 1
    cur.execute('INSERT INTO '+database_constants.RESULT_TABLE_NAME+' VALUES(?,?,?,?,?,?,?)',
                 (self.current_result_id,
                  data[0],
                  data[1],
                  data[2],
                  data[3],
                  data[4],
                  data[5]
                 )
               )
    
    self.connection.commit()
    return self.current_result_id

  def get_core_result_data_length(self):
    return 4
  
  def add_core_result(self, data):
    if len(data) != self.get_core_result_data_length():
      self.panic("The length of the cpu core data array must be "+str(self.get_core_result_data_length())+". Found length "+str(len(data)))
    
    cur = self.connection.cursor()
    cur.execute('INSERT INTO '+database_constants.CPU_CORE_TABLE_NAME+' VALUES(?,?,?,?)',
                 (data[0],
                  data[1],
                  data[2],
                  data[3]
                  )
               )
    
    self.connection.commit()
  
  def get_shared_bus_data_length(self):
    return 4
  
  def add_shared_bus_result(self, data):
    if len(data) != self.get_shared_bus_data_length():
      self.panic("The length of the shared bus data array must be "+str(self.get_shared_bus_data_length())+". Found length "+str(len(data)))
    cur = self.connection.cursor()
    self.current_shared_bus_id = self.current_shared_bus_id + 1
    cur.execute('INSERT INTO '+database_constants.BUS_TABLE_NAME+' VALUES(?,?,?,?,?)',
                 (self.current_shared_bus_id,
                  data[0],
                  data[1],
                  data[2],
                  data[3]
                  )
               )
    
    self.connection.commit()
    
  def get_interconnect_data_length(self):
    return 11
  
  def add_interconnect_result(self, data):
        if len(data) != self.get_interconnect_data_length():
            self.panic("The length of the interconnect data array must be "+str(self.get_interconnect_data_length())+". Found length "+str(len(data)))
        cur = self.connection.cursor()
        self.current_interconnect_id = self.current_interconnect_id + 1
        cur.execute('INSERT INTO '+database_constants.INTERCONNECT_TABLE_NAME+' VALUES(?,?,?,?,?,?,?,?,?,?,?,?)',
                 (self.current_interconnect_id,
                  data[0],
                  data[1],
                  data[2],
                  data[3],
                  data[4],
                  data[5],
                  data[6],
                  data[7],
                  data[8],
                  data[9],
                  data[10]
                  )
               )
        
        self.connection.commit()
  
  def get_shared_L2_data_length(self):
    return 2
  
  def add_shared_L2_result(self, data):
    if len(data) != self.get_shared_L2_data_length():
      self.panic("The length of the shared L2 cache data array must be "+str(self.get_shared_L2_data_length())+". Found length "+str(len(data)))
    cur = self.connection.cursor()
    self.current_L2_shared_id = self.current_L2_shared_id + 1
    cur.execute('INSERT INTO '+database_constants.L2_SHARED_TABLE_NAME+' VALUES(?,?,?)',
                 (self.current_L2_shared_id,
                  data[0],
                  data[1]
                  )
               )
    
    self.connection.commit()
  
  def get_L1i_data_length(self):
    return 3
  
  def add_l1_i_result(self, data):
    if len(data) != self.get_L1i_data_length():
      self.panic("The length of the L1 instruction cache data array must be "+str(self.get_L1i_data_length())+". Found length "+str(len(data)))
    
    cur = self.connection.cursor()
    self.current_L1_i_id = self.current_L1_i_id + 1
    cur.execute('INSERT INTO '+database_constants.L1I_TABLE_NAME+' VALUES(?,?,?,?)',
                 (self.current_L1_i_id,
                  data[0],
                  data[1],
                  data[2]
                  )
               )
    
    self.connection.commit()
    
  def get_L1d_data_length(self):
    return 7
  
  def add_l1_d_result(self, data):
    if len(data) != self.get_L1d_data_length():
      self.panic("The length of the L1 data cache data array must be "+str(self.get_L1d_data_length())+". Found length "+str(len(data)))
    
    cur = self.connection.cursor()
    self.current_L1_d_id = self.current_L1_d_id + 1
    cur.execute('INSERT INTO '+database_constants.L1D_TABLE_NAME+' VALUES(?,?,?,?,?,?,?,?)',
                 (self.current_L1_d_id,
                  data[0],
                  data[1],
                  data[2],
                  data[3],
                  data[4],
                  data[5],
                  data[6]
                  )
               )
    
    self.connection.commit()
    
  def panic(self, message):
    print 'ERROR (database.py):\t'+message
    sys.exit(-1)
    
