#!/usr/bin/python

import sys
import os

import pbsconfig as config
import boxplotter
import barplotter
import retrieve_plot_data as rpd
import plot_constants as pc

# Plot Utility Methods =========================================================

def get_data(retriever, data_type, cpu_count, normalise_index, average_type):
    if data_type == pc.PLOT_IPC:
        data = retriever.get_IPC(cpu_count, average_type)
    
    elif data_type == pc.PLOT_AVG_L1_MISS_RATE:
        data = retriever.get_average_L1_cache_miss_rate(cpu_count, average_type)
    
    elif data_type == pc.PLOT_SIM_TIME:
        data = retriever.get_simulation_time(cpu_count, average_type)
    
    elif data_type == pc.PLOT_SIM_TICKS:
        data = retriever.get_simulated_clock_cycles(cpu_count, average_type)
    
    elif data_type == pc.PLOT_CC_READ_INVALID:
        data = retriever.get_cc_read_invalid_misses(cpu_count, average_type)
        
    elif data_type == pc.PLOT_CC_WRITE_INVALID:
        data = retriever.get_cc_write_invalid_misses(cpu_count, average_type)
        
    elif data_type == pc.PLOT_CC_WRITE_OWNED:
        data = retriever.get_cc_write_owned_misses(cpu_count, average_type)
        
    elif data_type == pc.PLOT_CC_WRITE_SHARED:
        data = retriever.get_cc_write_shared_misses(cpu_count, average_type)
    
    elif data_type == pc.PLOT_BUS_DATA:
        data = retriever.get_bus_data_idle_fraction(cpu_count, average_type)
        
    elif data_type == pc.PLOT_BUS_ADDRESS:
        data = retriever.get_bus_address_idle_fraction(cpu_count, average_type)
        
    elif data_type == pc.PLOT_INTERCONNECT_CYCLES_PER_REQ:
        data = retriever.get_interconnect_cycles_per_req(cpu_count, average_type)
    
    else:
        print "WARNING (plotbench.py):\tUnknown data type to use in plotting"
        sys.exit(-1)
    
    if normalise_index >= 0:
        retriever.normalise(data, normalise_index)
    return data

def get_filename(data_name, cpu_count, filetype):
    
    if data_name in pc.plotnames:
        name = pc.plotnames[data_name]
    else:
        print "WARNING (plotbench.py):\tUnknown data constant used, consult plot_constants.py"
        name = "unknown"

    return config.experimentname+"_"+name+"_"+str(cpu_count)+"."+filetype

def set_plot_area(plotter, plotarea, cpu_count):
    if plotarea == None:
        plotter.min_y, plotter.max_y = "*","*"
    elif -1 in plotarea:
        plotter.min_y, plotter.max_y = plotarea[-1]
    else:
        plotter.min_y, plotter.max_y = plotarea[cpu_count]

def set_text_width(plotter, width):
    plotter.textwidth, plotter.datawidth = width
    
# Main Script ==================================================================

plot_dir = "plots"
if not os.access(plot_dir, os.F_OK):
    os.mkdir(plot_dir)
    print "Plots directory created..."
print "Plotting results..."
for plot in config.plots:
    
    type = plot[0]
    data_type = plot[1]
    average_type = plot[2]
    title, y_title, x_tile = plot[3]
    textwidth = plot[4]
    plotarea = plot[5]
    normalise_index = plot[6]
    
    db_retriever = rpd.Retrieve_Plot_Data(config.benchmarks, config.protocols, config.interconnects)
    
    if type == pc.BOX_PLOT:
        
        boxpl = boxplotter.Box_Plotter(title, x_tile, y_title, config.benchmarks, config.protocols, plot_dir)
        
        if textwidth != None:
            set_text_width(boxpl, textwidth)
        
        for cpu_count in config.number_of_cpus:
            
            print "Creating BoxPlot "+title+" with "+str(cpu_count)+" cpus with GnuPlot"
            
            set_plot_area(boxpl, plotarea, cpu_count)
            
            data_file_name = get_filename(data_type, cpu_count, "data")
            gp_file_name = get_filename(data_type, cpu_count, "gp")
            eps_file_name = get_filename(data_type, cpu_count, "eps")
            
            experiment_data = get_data(db_retriever, data_type, cpu_count, normalise_index, average_type)
            
            boxpl.write_data_file(data_file_name, experiment_data)
            boxpl.write_benchmark_script(data_file_name, gp_file_name, eps_file_name, cpu_count)
            boxpl.plot_boxes(gp_file_name, eps_file_name, jpg_file_name)
    
    elif type == pc.BAR_PLOT:
        barpl = barplotter.Bar_Plotter(title, x_tile, y_title, config.benchmarks, config.protocols, plot_dir)
        
        for cpu_count in config.number_of_cpus:
            print "Creating BarPlot "+title+" with "+str(cpu_count)+" cpus with R"
            
            set_plot_area(barpl, plotarea, cpu_count)
            
            csv_file_name = get_filename(data_type, cpu_count, "csv")
            r_file_name = get_filename(data_type, cpu_count, "r")
            eps_file_name = get_filename(data_type, cpu_count, "eps")
            
            experiment_data = get_data(db_retriever, data_type, cpu_count, normalise_index, average_type)
            
            barpl.write_data_file(experiment_data, csv_file_name)
            barpl.write_script(csv_file_name, r_file_name, eps_file_name, cpu_count)
            barpl.plot_bars(r_file_name, eps_file_name)
    
    # TODO: ADD OTHER PLOT TYPES HERE
    
    else:
        print "Unimplemented plot type"
print "Finished!"
