#!/usr/bin/python
import sys
import popen2
import config

# ----------- Method declarations -----------

def plotNumBytes(inFilename, cpu_count, plotConfig):
  
  gpFilename = config.getPlotOrParseFilename(pAndgConfig[0], cpu_count)+".gp"
  epsFilename = config.getPlotOrParseFilename(pAndgConfig[0], cpu_count)+".eps"

  output = open(gpFilename,'w')
  
  output.write("set title \""+plotConfig[1]+" with "+str(cpu_count)+" CPUs\"\n");
  output.write("set xlabel \""+plotConfig[2]+"\"\n");
  output.write("set ylabel \""+plotConfig[3]+"\"\n");
  if plotConfig[4] > 0:
    output.write("set xr [0:"+str(plotConfig[4])+"]\n");
  if plotConfig[5] > 0:
    output.write("set yr ["+str(plotConfig[7])+":"+str(plotConfig[5])+"]\n");
  
  output.write("set terminal postscript eps color enhanced 18\n");
  output.write("set output \""+epsFilename+"\"\n");
  
  print "Generating file "+epsFilename
  
  output.write("plot ")
  for i in range(0, len(config.configurations)):
    currentConfig = config.configurations[i]
    if plotConfig[6].count(i) > 0 or plotConfig[6] == []:
      output.write("\""+inFilename+"\" using 1:"+str(i+2)+" title \"\
"+currentConfig[0]+"\" with "+plotConfig[0])
      
      if i == (len(config.configurations)-1) or i == (len(plotConfig[6])-1):
        output.write("\n")
      else:
        output.write(", \\\n")
        
  
  output.close()
  
  results = popen2.popen3("gnuplot "+ gpFilename)
  print "Gnuplot output: "+results[0].readline()
  
  results = popen2.popen3("epstopdf "+ epsFilename)
  print "epstopdf output: "+results[0].readline()

def plotBoxes(inFilename, cpu_count, plotConfig):
  gpFilename = config.getPlotOrParseFilename(pAndgConfig[0], cpu_count)+".gp"
  epsFilename = config.getPlotOrParseFilename(pAndgConfig[0], cpu_count)+".eps"

  output = open(gpFilename,'w')
  
  output.write("set title \""+plotConfig[1]+" with "+str(cpu_count)+" CPUs\"\n");
  output.write("set xlabel \""+plotConfig[2]+"\"\n");
  output.write("set ylabel \""+plotConfig[3]+"\"\n");
    
  output.write("set terminal postscript eps color enhanced\n");
  output.write("set output \""+epsFilename+"\"\n");
  
  output.write("set xtics rotate (")
  i = 0
  for numb, numt, option in config.pingpongParameters:
    output.write("\""+str(numb)+"\" "+str(i))
    if i == (len(config.pingpongParameters)-1):
      output.write(")\n\n")
    else:
      output.write(", ")
    i = i + 1
  
  print "Generating file "+epsFilename
  
  boxwidth = 0.8 / len(config.configurations)
  startpoint = 0 - ((len(config.configurations)/2) * boxwidth)
  
  output.write("plot ")
  if plotConfig[4] > 0:
    output.write("[*:"+str(plotConfig[4])+"] ")
  else:
    output.write("[*:*] ")
  if plotConfig[5] > 0:
    output.write("[0:"+str(plotConfig[5])+"] ")
  else:
    output.write("[0:*] ")
  
  for i in range(0, len(config.configurations)):
    currentConfig = config.configurations[i]
    if plotConfig[6].count(i) > 0 or plotConfig[6] == []:
      
      thisStart = startpoint + (i*boxwidth)
      if thisStart < 0:
        thisStartStr = str(thisStart)
      else:
        thisStartStr = "+"+str(thisStart)
      
      output.write("\""+inFilename+"\" ")
      output.write("using ($2"+thisStartStr+"):"+str(i+3)+":("+str(boxwidth)+") ")
      output.write("title \""+currentConfig[0]+"\" ")
      output.write("with boxes fs fill pattern "+str(i+1))
      
      if i == (len(config.configurations)-1) or i == (len(plotConfig[6])-1):
        output.write("\n")
      else:
        output.write(", \\\n")
        
  
  output.close()
  
  results = popen2.popen3("gnuplot "+ gpFilename)
  print "Gnuplot output: "+results[0].readline()
  
  results = popen2.popen3("epstopdf "+ epsFilename)
  print "epstopdf output: "+results[0].readline()
  
def plotTransactions():
  cpus = [2,4,6,8]
  
  for cpu_count in cpus:
    gpFilename = "transactions_"+str(cpu_count)+"cpu.gp"
    epsFilename = "transactions_"+str(cpu_count)+"cpu.eps"
    
    offsetDict = {2:2, 4:7, 6:12, 8:17}
    names = ["4KB L1", "8KB L1", "16KB L1", "32KB L1", "64KB L1"]
  
    output = open(gpFilename,'w')
    
    output.write("set title \"Finished transactions with "+str(cpu_count)+" CPUs\"\n");
    output.write("set xlabel \"Transaction size (bytes)\"\n");
    output.write("set ylabel \"Average number of transactions\"\n");
    #output.write("set xr [0:"+str(plotConfig[4])+"]\n");
    #output.write("set yr [0:"+str(plotConfig[5])+"]\n");
    
    output.write("set terminal postscript eps color enhanced 18\n");
    output.write("set output \""+epsFilename+"\"\n");
    
    print "Generating file "+epsFilename
    
    output.write("plot ")
    i = 0
    for name in names:
      output.write("\"transactions.dat\" using 1:"+str(offsetDict[cpu_count]+i)+"\
 title \""+name+"\" with linespoints")
      
      if i == (len(names)-1):
        output.write("\n")
      else:
        output.write(", \\\n")
    
      i = i + 1
          
    
    output.close()
    
    results = popen2.popen3("gnuplot "+ gpFilename)
    print "Gnuplot output: "+results[0].readline()
    
    results = popen2.popen3("epstopdf "+ epsFilename)
    print "epstopdf output: "+results[0].readline()

# ----------- Main script -----------

for pAndgConfig in config.parseAndGraph:
  
  plotConfig = pAndgConfig[2]
  
  for cpu_count in config.numberOfCpus:
    inFilename = config.getPlotOrParseFilename(pAndgConfig[0], cpu_count)+".dat"
  
    if pAndgConfig[4] == "NumBytes":
      print "Generating NumBytes plot"
      plotNumBytes(inFilename, cpu_count, plotConfig)
    elif pAndgConfig[4] == "Boxes":
      print "Generating Boxes plot"
      plotBoxes(inFilename, cpu_count, plotConfig)
    else:
      print "Unknown graphtype"

plotTransactions()

