#!/usr/bin/python

import re

import pbsconfig
import database
import database_constants as dc

BUS_INTERCONNECT_NAME = 'bus'

class Process_results:
  resultdb = None
  results = None
  all_bm_keys = {}
  all_config_keys = {}
  resultdb = None
  
  def __init__(self, resultdb):
    self.resultdb = resultdb

  def make_config_id(self, np, protocol, interconnect):
    return str(np)+'.'+protocol+'.'+interconnect
  
  def add_benchmark(self, name):
    dbid = self.resultdb.add_benchmark([name])
    self.all_bm_keys[name] = dbid
  
  def add_configuration(self, np, protocol, interconnect):
    dbid = self.resultdb.add_configuration([np, protocol, interconnect, '', '', ''])
    self.all_config_keys[self.make_config_id(np,protocol, interconnect)] = dbid
  
  def find_patterns(self, file, np):
    self.results = {}
    patterns = pbsconfig.get_patterns(np)
    
    line = file.readline()
    while line:
      tokens = re.split('\s+', line)
      
      if patterns.has_key(tokens[0]):
        if patterns[tokens[0]].has_key("all"): # Results relevant for all cores
          table, field_num = patterns[tokens[0]]["all"]
          if table not in self.results:
            self.results[table] = {}
          self.results[table][field_num] = tokens[1]
          
        else: # Per-core results
          cpu_id = patterns[tokens[0]].keys()[0]
          table, field_num = patterns[tokens[0]][cpu_id]

          if table not in self.results:
            self.results[table] = {}

          if cpu_id not in self.results[table]:
            self.results[table][cpu_id] = {}

          self.results[table][cpu_id][field_num] = tokens[1]
        
      line = file.readline()
    #print self.results
  
  def process_results(self, benchmark, np, protocol, interconnect):
    
    if self.results != {}:
        # data for result table
        this_result_ID = self.add_result(benchmark, np, protocol, interconnect)
        this_core_ID = self.add_core_result(this_result_ID)
        #FIXME: add L2 cache result processing
        #self.add_L2_result(this_result_ID)
        self.add_L1_i_result(this_result_ID)
        self.add_L1_d_result(this_result_ID)
      
        # interconnects get special treatment
        if interconnect == BUS_INTERCONNECT_NAME:
            self.add_shared_bus_result(this_result_ID)
        else:
            self.add_interconnect_result(this_result_ID)

    else:
      self.warning("Found no results for experiment "+str(np)+"."+str(benchmark)+"."+protocol+"."+interconnect)
  
  
  def add_result(self, benchmark, np, protocol, interconnect):
    
    result_tuple = self.results[dc.RESULT_TABLE_NAME]
    keylist = [self.all_config_keys[self.make_config_id(np,protocol,interconnect)],
               self.all_bm_keys[benchmark]]
    data = self.create_data_list(keylist, result_tuple, self.resultdb.get_result_data_length(), "Result")
    
    return self.resultdb.add_result(data)

  def add_core_result(self, result_ID):
    result_tuples = self.results[dc.CPU_CORE_TABLE_NAME]
    for cpu_id in result_tuples.keys():
      result_tuple = result_tuples[cpu_id]
      data = self.create_data_list([cpu_id, result_ID], result_tuple, self.resultdb.get_core_result_data_length(), "CPU core "+str(cpu_id))
      self.resultdb.add_core_result(data)
  
  def add_shared_bus_result(self, result_ID):
    result_tuple = self.results[dc.BUS_TABLE_NAME]
    data = self.create_data_list([result_ID], result_tuple, self.resultdb.get_shared_bus_data_length(), "Shared bus")
    self.resultdb.add_shared_bus_result(data)
  
  def add_interconnect_result(self, result_ID):
    result_tuple = self.results[dc.INTERCONNECT_TABLE_NAME]
    data = self.create_data_list([result_ID], result_tuple, self.resultdb.get_interconnect_data_length(), "Interconnect")
    self.resultdb.add_interconnect_result(data)
  
  def add_L2_result(self, result_ID):
    if pbsconfig.shared_L2:
      # Private L2 configuration
      result_tuple = self.results[dc.L2_SHARED_TABLE_NAME]
      data = self.create_data_list([result_ID], result_tuple, self.resultdb.get_shared_L2_data_length(), "Shared L2 cache")
      self.resultdb.add_shared_L2_result(data)
    else:
        #TODO: implement private cache parsing
        warning("Private L2 cache result parsing not implemented")
  
  def add_L1_i_result(self, this_result_ID):
    result_tuples = self.results[dc.L1I_TABLE_NAME]
    for cpu_id in result_tuples.keys():
      result_tuple = result_tuples[cpu_id]
      data = self.create_data_list([cpu_id, this_result_ID], result_tuple, self.resultdb.get_L1i_data_length(),"L1 instruction cache (cpu "+str(cpu_id)+")")
      self.resultdb.add_l1_i_result(data)
    
  def add_L1_d_result(self, this_result_ID):
    result_tuples = self.results[dc.L1D_TABLE_NAME]
    for cpu_id in result_tuples.keys():
      result_tuple = result_tuples[cpu_id]
      data = self.create_data_list([cpu_id, this_result_ID], result_tuple, self.resultdb.get_L1d_data_length(),"L1 data cache (cpu "+str(cpu_id)+")")
      self.resultdb.add_l1_d_result(data)
    
  def create_data_list(self, IDs, results, data_length, origin):
    
    data = []
    for ID in IDs:
      data.append(ID)
    
    not_found = []
    for i in range(len(IDs), data_length):
      if i in results:
        data.insert(i, results[i])
      else:
        data.insert(i, -1)
        not_found.append(i)
    
    
    if not_found != []:
      self.warning(origin+": could not find element(s) "+str(not_found))
    
    return data
  
  def warning(self, message):
    print "WARNING (process_results.py):\t"+message

  
  
  
  
