
from pysqlite2 import dbapi2 as sqlite

import database_constants as dc
import plot_constants as pc

# === Retrieve_Plot_Data Class =================================================

class Retrieve_Plot_Data:

    PER_MILL_INSTRUCTION = False
  
    def __init__(self, benchmarks, protocols, interconnects):
        self.connection = sqlite.connect(dc.DATABASE_FILE)
        self.benchmarks = benchmarks
        self.protocols = protocols
        self.interconnects = interconnects
        
        # Create config dict
        self.config_keys = {}
        config_query = "SELECT * FROM configuration"
        cur = self.connection.cursor()
        cur.execute(config_query)
        
        for fields in cur:
            self.config_keys[self.create_hash(fields[1:])] = fields[0]
        
        # Create benchmark dict
        self.benchmark_keys = {}
        benchmark_query = "SELECT * FROM benchmark"
        cur = self.connection.cursor()
        cur.execute(benchmark_query)
        
        for fields in cur:
            self.benchmark_keys[self.create_hash(fields[1:])] = fields[0]
    
    def get_IPC(self, cpu_count, average_type):
        query = "SELECT c.ipc, c.cpu_id FROM benchmark b, configuration conf, result r, cpu_core_result c WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
    
    def get_average_L1_cache_miss_rate(self, cpu_count, average_type):
        query = "SELECT d.overall_miss_rate, d.cpu_core_result_id FROM benchmark b, configuration conf, result r, cpu_core_result c, L1_data_cache_result d WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND d.result_id = c.result_id  AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
        
    def get_simulation_time(self, cpu_count, average_type):
        query = "SELECT r.host_simulation_time FROM benchmark b, configuration c, result r WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND b.id = ? AND c.id = ?;"
        data = self.get_db_data(query, cpu_count, average_type)
        
        # convert to minutes
        minute_data = []
        for line in data:
            name, vals = line
            minute_vals = []
            for val in vals:
                val = (val / 60)
                minute_vals.append(val)
            minute_data.append((name, minute_vals))
        
        return minute_data
            
    def get_simulated_clock_cycles(self, cpu_count, average_type):
        query = "SELECT r.simulated_ticks FROM benchmark b, configuration c, result r WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND b.id = ? AND c.id = ?;"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
        
    def get_cc_read_invalid_misses(self, cpu_count, average_type):    
        self.PER_MILL_INSTRUCTION = True
        query = "SELECT d.cc_read_invalid, d.cpu_core_result_id FROM benchmark b, configuration conf, result r, cpu_core_result c, L1_data_cache_result d WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND d.result_id = c.result_id  AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        self.PER_MILL_INSTRUCTION = False
        return data
        
    def get_cc_write_invalid_misses(self, cpu_count, average_type):
        self.PER_MILL_INSTRUCTION = True
        query = "SELECT d.cc_write_invalid, d.cpu_core_result_id FROM benchmark b, configuration conf, result r, cpu_core_result c, L1_data_cache_result d WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND d.result_id = c.result_id  AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        self.PER_MILL_INSTRUCTION = False
        return data
        
    def get_cc_write_owned_misses(self, cpu_count, average_type):
        self.PER_MILL_INSTRUCTION = True
        query = "SELECT d.cc_write_owned, d.cpu_core_result_id FROM benchmark b, configuration conf, result r, cpu_core_result c, L1_data_cache_result d WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND d.result_id = c.result_id  AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        self.PER_MILL_INSTRUCTION = False
        return data
    
    def get_cc_write_shared_misses(self, cpu_count, average_type):
        self.PER_MILL_INSTRUCTION = True
        query = "SELECT d.cc_write_shared, d.cpu_core_result_id FROM benchmark b, configuration conf, result r, cpu_core_result c, L1_data_cache_result d WHERE conf.id = r.configuration_id AND b.id = r.benchmark_id AND c.result_id = r.id AND d.result_id = c.result_id  AND b.id = ? AND conf.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        self.PER_MILL_INSTRUCTION = False
        return data

    def get_bus_data_idle_fraction(self, cpu_count, average_type):
        query = "SELECT s.data_idle_fraction FROM benchmark b, configuration c, result r, shared_bus_result s WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND r.id = s.result_id AND b.id = ? And c.id = ?;"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
        
    def get_bus_address_idle_fraction(self, cpu_count, average_type):
        query = "SELECT s.address_idle_fraction FROM benchmark b, configuration c, result r, shared_bus_result s WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND r.id = s.result_id AND b.id = ? And c.id = ?;"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
        
    def get_interconnect_cycles_per_req(self, cpu_count, average_type):
        #data = self.get_interconnect_data(cpu_count, average_type)
        query = "SELECT i.avgTotalDelayCyclesPerReq FROM benchmark b, configuration c, result r, interconnect_result i WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND r.id = i.result_id AND b.id = ? And c.id = ?"
        data = self.get_db_data(query, cpu_count, average_type)
        return data
    
# === Utility Method(s) ========================================================
    
    def get_db_data(self, query, cpu_count, average_type):
        data = []
        benchnum = 0
        for benchmark in self.benchmarks:
            data.insert(benchnum, (benchmark, []))
            benchID = self.benchmark_keys[self.create_hash([benchmark])]
            confignum = 1
            for protocol in self.protocols:
                for interconnect in self.interconnects:
                    confID = self.config_keys[self.create_hash([cpu_count, protocol, interconnect])]
                    query_result = self.execute_query(query, (benchID, confID)) 
                    result_list = self.get_value_list(query_result, 0)
                    value = self.compute_average(result_list, average_type)
                
                    if self.PER_MILL_INSTRUCTION:
                        inst_count = self.get_instruction_count(benchID, confID)
                        if inst_count > 0:
                            value = float(float(value) / inst_count)
                        else:
                            value = 0
                    
                    data[benchnum][1].insert(confignum, value)
                    confignum = confignum+1
                
            benchnum = benchnum + 1 
        
        return data
    

    def execute_query(self, query, query_vars):
        result = []
        
        cur = self.connection.cursor()
        cur.execute(query, query_vars)
        
        for fields in cur:
          result.append(fields)
          
        return result

    def normalise(self, data, normalise_index):
        new_data = []
        for line in data:
            name, vals = line
            normalise_val = vals[normalise_index]
            
            if normalise_val == 0:
                self.warning("Value to normalise against is 0. Normalisation will not be performed...")
                continue
            
            for i in range(0, len(vals)):
                vals[i] = (vals[i] / normalise_val)
            
            line = (name, vals)
    
    def get_value_list(self, query_result, index):
        list = []
        i = 0
        for element in query_result:
            list.insert(i, element[index])
            i = i +1
        return list
      
    # Concatenates all values in a database row into one string (assumed to be unique)
    # This is used to identify keys for a given configuration or benchmark
    def create_hash(self, inlist):
        hash = ""
        for element in inlist:
            hash = hash+str(element)
        return hash
    
    def compute_average(self, list, average_type):
        if average_type == pc.HARMONIC_MEAN:
            return self.harmonic_mean(list)
        elif average_type == pc.ARITHMETIC_MEAN:
            return self.arithmetic_mean(list)
        elif average_type == None:
            if list == []:
                return 0
            else:
                return list[0]
        else:
            self.warning("Unknown average type in compute_average")
            return 0
    
    def harmonic_mean(self, list):
        invsum = 0.0
        for element in list:
            float_elem = 0 
            try:
                float_elem = float(element)
            except ValueError:
                self.warning("Supplied value to harmonic mean wasn't a number, result set to 0")
                return 0

            if float_elem == 0:
                self.warning("An element in a harmonic mean calculation was 0, returning 0")
                return 0
            
            invsum = invsum + (1/float_elem)
        
        if invsum == 0:
            self.warning("Divide by zero in harmonic mean method, result set to 0")
            return 0
        
        return (len(list) / invsum)
    
    def arithmetic_mean(self, list):
        if len(list) == 0:
            self.warning("Divide by zero in arithmetic mean method, result set to 0")
            return 0
        
        sum = 0.0
        for element in list:
            try:
                sum = sum + float(element)
            except ValueError:
                self.warning("Supplied value to arithmetic mean wasn't a nunber, result set to 0")
                return 0
        
        result = sum / len(list)
        
        # -1 is the not found value in the database. The user has allready been
        # warned when the DB was created so there is no need to do it again
        if result == -1:
            return 0
        else:
            return result
        
        
    def get_instruction_count(self, benchID, confID):
        query_result = self.execute_query("SELECT simulated_instructions FROM result WHERE benchmark_id=? AND configuration_id=?", (benchID, confID))
        result_list = self.get_value_list(query_result, 0)
        if result_list != []: 
            inst_count = result_list[0]
            mill_insts = float(float(inst_count) / 1000000)
            return mill_insts
        else:
            self.warning("No instruction count found, returning -1")
            return float(-1)

    def warning(self, message):    
        print 'WARNING (retrieve_plot_data.py):\t'+message


#====== DEPRECATED ============================================================
    #def get_interconnect_data(self, cpu_count, average_type):
        #data = []
        #benchnum = 0
        #for benchmark in self.benchmarks:
            #data.insert(benchnum, (benchmark, []))
            #benchID = self.benchmark_keys[self.create_hash(benchmark)]
            #confignum = 1
            #for protocol in self.protocols:
                #for interconnect in self.interconnects:
                    #confID = self.config_keys[self.create_hash([cpu_count, protocol, interconnect])]
                    
                    ##print "experiment "+self.create_hash([cpu_count, protocol, interconnect])
                    
                    #query = ''
                    #if interconnect == 'bus':
                        #query = "SELECT s.data_use_cycles FROM benchmark b, configuration c, result r, shared_bus_result s WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND r.id = s.result_id AND b.id = ? And c.id = ?;"
                    #else:
                        #query = "SELECT cr.lineUseCycles FROM benchmark b, configuration c, result r, crossbar_result cr WHERE b.id = r.benchmark_id AND c.id = r.configuration_id AND r.id = cr.result_id AND b.id = ? And c.id = ?;" 
                    
                    #query_result = self.execute_query(query, (benchID, confID)) 
                    #result_list = self.get_value_list(query_result, 0)
                    #value = self.compute_average(result_list, average_type)
                    
                    ##print value
                    
                    #if self.PER_MILL_INSTRUCTION:
                        #inst_count = self.get_instruction_count(benchID, confID)
                        #if inst_count > 0:
                            #value = float(float(value) / inst_count)
                        #else:
                            #value = 0
                    ##print "inserting at "+str(confignum)
                    #data[benchnum][1].insert(confignum, value)
                    #confignum = confignum+1
                    
            ##print data
            #benchnum = benchnum + 1
        #return data
